//$Id: CfgIbs.cpp 15320 2008-09-30 14:30:43Z ssuthiku $

/*
// CodeAnalyst for Open Source
// Copyright 2002 - 2008 Advanced Micro Devices, Inc.
// You may redistribute this program and/or modify this program under the terms
// of the GNU General Public License as published by the Free Software 
// Foundation; either version 2 of the License, or (at your option) any later 
// version.
//
// This program is distributed WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED 
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  See the 
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "CfgIbs.h"
#include "iruncontrol.h"
#include "atuneoptions.h"
#include <qbuttongroup.h>
#include "xp.h"

IbsCfgDlg::IbsCfgDlg ( QWidget* parent, const char* name, bool modal, WFlags fl) 
: IIbsCfg (parent, name, modal, fl)
{
	m_pProfiles = NULL;
	m_dispatchAvail = false;

	connect (buttonOk, SIGNAL (clicked ()), SLOT (onOk ()));
	connect (m_pProfileName, SIGNAL (textChanged ( const QString &)), 
		SLOT (onModified ()));
	connect (m_pFetchInterval, SIGNAL (textChanged ( const QString &)), 
		SLOT (onModified ()));
	connect (m_pOpInterval, SIGNAL (textChanged ( const QString &)), 
		SLOT (onModified ()));
	connect (m_pFetch, SIGNAL (clicked ()), SLOT (onModified ()));
	connect (m_pOp, SIGNAL (clicked ()), SLOT (onModified ()));
	connect (m_pOpMode, SIGNAL (clicked (int)), SLOT (onModified ()));

	connect (m_pFetch, SIGNAL (toggled ( bool )), m_pFetchLabel,
		SLOT (setEnabled ( bool )));
	connect (m_pFetch, SIGNAL (toggled ( bool )), m_pFetchInterval,
		SLOT (setEnabled ( bool )));
	connect (m_pOp, SIGNAL (toggled ( bool )), m_pOpLabel,
		SLOT (setEnabled ( bool )));
	connect (m_pOp, SIGNAL (toggled ( bool )), m_pOpInterval,
		SLOT (setEnabled ( bool )));
	connect (m_pOp, SIGNAL (toggled ( bool )), m_pOpCycle,
		SLOT (setEnabled ( bool )));
	connect (m_pOp, SIGNAL (toggled ( bool )), m_pOpMode,
		SLOT (setEnabled ( bool )));
	connect (m_pOp, SIGNAL (toggled ( bool )), this,
		SLOT (onOpEnabled ( bool )));
}

IbsCfgDlg::~IbsCfgDlg ()
{
}


void IbsCfgDlg::onModified ()
{
	m_modified = true;
}

void IbsCfgDlg::onOpEnabled (bool enabled)
{
	m_pOpDispatch->setEnabled (enabled && m_dispatchAvail);
}

void IbsCfgDlg::setProperties (IBS_OPTIONS *pSession)
{
	setCaption ("Review Ibs configuration");
	//add run control
	IRunControl * pRunControl = new IRunControl (this);
	RETURN_IF_NULL (pRunControl, this);
	setExtension (pRunControl);
	setOrientation (Vertical);
	showExtension (true);

	//set readonly
	m_pProfileName->setReadOnly (true);
	m_pFetchInterval->setReadOnly (true);
	m_pFetch->setEnabled (false);
	m_pOpInterval->setReadOnly (true);
	m_pOp->setEnabled (false);
	m_pLaunch->setReadOnly (true);
	m_pWorkDir->setReadOnly (true);
	pRunControl->m_pDuration->setReadOnly (true);
	pRunControl->m_pStartDelay->setReadOnly (true);
	pRunControl->m_pStartPaused->setEnabled (false);
	pRunControl->m_pAffinityValue->setReadOnly(true);	
	pRunControl->m_pAffinityBtn->hide();
	pRunControl->m_pApplyFilter->setEnabled(false);

	//hide cancel
	buttonCancel->hide();

	//Add setting info
	m_pProfileName->setText (pSession->sessionName);
	m_pFetchInterval->setText (QString::number (pSession->fetchInterval));
	m_pFetch->setChecked (pSession->fetchSample);
	m_pOpInterval->setText (QString::number (pSession->opInterval));
	m_pOp->setChecked (pSession->opSample);
	m_pOpCycle->setChecked (pSession->opCycleCount);
	m_pOpCycle->setEnabled (pSession->opCycleCount);
	m_pOpDispatch->setChecked (!pSession->opCycleCount);
	m_pOpDispatch->setEnabled (!pSession->opCycleCount);

	//Run control info
	pRunControl->m_pDuration->setText ((pSession->duration <= 0)? "N/A": QString::number( pSession->duration));
	pRunControl->m_pProfileDuration->setChecked (0 == pSession->duration);
	pRunControl->m_pStartDelay->setText ((pSession->startDelay < 0)? "N/A": QString::number( pSession->startDelay));
	pRunControl->m_pStartPaused->setChecked (pSession->startPaused);
	pRunControl->m_pStopOnExit->setChecked (pSession->stopOnAppExit);
	pRunControl->m_pTerminateAfter->setChecked (pSession->termAppOnDur);
	QString afStr; 

	if(pSession->affinity == 0)
		afStr = QString("N/A");
	else
		afStr = QString("0x") + QString::number(pSession->affinity,16);

	pRunControl->m_pAffinityValue->setText(afStr);

	m_pLaunch->setText (pSession->launch);
	m_pWorkDir->setText (pSession->workDir);
	m_modified = false;
} //IbsCfgDlg::setProperties


void IbsCfgDlg::setProfile (ProfileCollection *pProfiles, QString name)
{
	m_profileName = name;
	m_pProfiles = pProfiles;
	IBS_OPTIONS ibsOptions;
	m_pProfiles->getProfileConfig (name, &ibsOptions);
	m_pProfileName->setText (m_profileName);
	m_pFetchInterval->setText (QString::number (ibsOptions.fetchInterval));
	m_pOpInterval->setText (QString::number (ibsOptions.opInterval));
	m_pFetch->setChecked (ibsOptions.fetchSample);
	m_pOp->setChecked (ibsOptions.opSample);

	// REV C support
	char VendorId[15];
	unsigned long Family;
	unsigned long Model;
	unsigned long Stepping;
	unsigned long Features;
	CpuId( VendorId, &Family, &Model, &Stepping, &Features );

	if ((GREYHOUND_FAMILY== Family) && (REV_C_MODEL <= Model))
	{
		m_dispatchAvail = true;
		m_pOpCycle->setEnabled (true);
		m_pOpCycle->setChecked (ibsOptions.opCycleCount);
		m_pOpDispatch->setEnabled (true);
		m_pOpDispatch->setChecked (!ibsOptions.opCycleCount);
	} else {
		m_dispatchAvail = false;
		m_pOpCycle->setEnabled (true);
		m_pOpCycle->setChecked (true);
		m_pOpDispatch->setEnabled (false);
		m_pOpDispatch->setChecked (false);
	}

	m_pLaunchBox->hide();
	m_modified = false;
}

void IbsCfgDlg::onOk ()
{
	if (m_pProfileName->text().isEmpty())
	{
		QMessageBox::question (this, "CodeAnalyst error",
			"Please specify profile name.");
		m_pProfileName->setFocus ();
		return;
	}

	if ((m_modified) && (NULL != m_pProfiles))
	{
		//If the name would overwrite an existing profile
		if ((m_profileName != m_pProfileName->text()) && 
			(NO_TRIGGER != m_pProfiles->getProfileType (m_pProfileName->text()))
			&& (QMessageBox::question (this, "CodeAnalyst warning",
			"There is already a profile with that name,"
			"\ndo you want to overwrite the profile?",
			QMessageBox::Yes, QMessageBox::No) == QMessageBox::No))
		{
			//Set the focus, so the user can change the name
			m_pProfileName->setFocus ();
			return;
		}

		// Check if either type is checked
		if ((!m_pFetch->isChecked ()) && (! m_pOp->isChecked ()))
		{
			QMessageBox::warning (this, "CodeAnalyst warning",
				"At least one sampling method needs to be selected for a valid profile.");
			m_pOp->setFocus ();
			return;
		}

		// Check Fetch Interval	
		if (m_pFetch->isChecked()
			&& (m_pFetchInterval->text().toULong() < CA_IBS_FETCH_MIN
			||  m_pFetchInterval->text().toULong() > CA_IBS_FETCH_MAX))
		{
			QString msg = QString("IBS Fetch Interval : ") 
				+ m_pFetchInterval->text() 
				+ " is not a valid.\n"
				+ "Please specify value between "
				+ QString::number(CA_IBS_FETCH_MIN)
				+ " and "
				+ QString::number(CA_IBS_FETCH_MAX)
				+ ".\n";
			QMessageBox::warning (this, "CodeAnalyst warning",msg);
			m_pFetch->setFocus ();
			return;
		}

		// Check Op Interval	
		if (m_pOp->isChecked()
			&& (m_pOpInterval->text().toULong() < CA_IBS_FETCH_MIN
			||  m_pOpInterval->text().toULong() > CA_IBS_FETCH_MAX))
		{
			QString msg = QString("IBS Op Interval : ")
				+ m_pOpInterval->text() 
				+ " is not a valid.\n"
				+ "Please specify value between "
				+ QString::number(CA_IBS_FETCH_MIN)
				+ " and "
				+ QString::number(CA_IBS_FETCH_MAX)
				+ ".\n";
			QMessageBox::warning (this, "CodeAnalyst warning",msg);
			m_pOp->setFocus ();
			return;
		}


		//Save modified profile
		IBS_OPTIONS ibsOptions;
		ibsOptions.fetchInterval = m_pFetchInterval->text().toULong ();
		ibsOptions.opInterval = m_pOpInterval->text().toULong ();
		ibsOptions.fetchSample = m_pFetch->isChecked ();
		ibsOptions.opSample = m_pOp->isChecked ();
		ibsOptions.opCycleCount = m_pOpCycle->isChecked ();
		if(!m_pProfiles->setProfileConfig (m_pProfileName->text(), &ibsOptions))
		{
			m_pProfileName->setFocus();
			return;
		}	
	}
	accept();
} //IbsCfgDlg::onOk

bool IbsCfgDlg::wasModified ()
{
	return m_modified;
}
